#!/usr/bin/env python
# Transform 'filband' and 'filproj' to Gnuplot format
# Davide Ceresoli, Sat Jan 24, 2015

import re, os, sys, argparse, numpy

#======================================================================
# Read the filband file and return ek(nks,nbnd)
#======================================================================
def read_filband(filband):
    f = open(filband)
    header = f.readline()
    m = re.match(" &plot nbnd=\s*(\d+), nks=\s*(\d+)", header)
    if m:
         nbnd, nks = int(m.group(1)), int(m.group(2))
         print "reading band file '%s': nks=%i, nbnd=%i" % (filband, nks, nbnd)
    else:
         print "error reading band file header:"
         print header.strip()
         sys.exit(1)
         
    ek = numpy.zeros((nks,nbnd))
    for ik in xrange(nks):
        f.readline()    # skip k-points
        data = []
        for i in xrange(int(nbnd/10)):
            data.extend(map(float, f.readline().split()))
        if nbnd % 10 != 0:
            data.extend(map(float, f.readline().split()))
        assert len(data) == nbnd
        ek[ik,:] = numpy.array(data)

    f.close()        
    return ek
    

#======================================================================
# Read the filproj file and return proj(nks,nbnd)
#======================================================================
def read_filproj(filproj, projlist):
    f = open(filproj)
    f.readline()
    natom, nsp = map(int, f.readline().split()[6:])
    ibrav = int(f.readline().split()[0])
    if ibrav == 0:
         f.readline(); f.readline(); f.readline()
    f.readline()
    for i in xrange(nsp+natom):
        f.readline()

    norb, nks, nbnd = map(int, f.readline().split())
    print "reading proj file '%s': norb=%i, nks=%i, nbnd=%i" % (filproj, norb, nks, nbnd)
    f.readline()  # F  F
         
    proj = numpy.zeros((nks,nbnd))
    for iorb in xrange(norb):
        f.readline()    # skip line
        if iorb+1 in projlist:
            for ik in xrange(nks):
                for i in xrange(nbnd):
                    line = f.readline().split()
                    ik_, i_ = map(int, line[0:2])
                    p = float(line[2])
                    assert ik+1 == ik_
                    assert i+1 == i_
                    proj[ik,i] += p
        else:
            for i in xrange(nks*nbnd):
                f.readline()
                  
    f.close()    
    return proj


#======================================================================
# Expand a string such as "1,5-2, 7,10" into a list of integers
#======================================================================
def expand_list(expr):
    numbers = []

    for num in expr.split(','):
        num = num.strip()
        num2 = num.split('-')
        if len(num2) == 1:            # single entry
           numbers.append(int(num))
        elif len(num2) == 2:          # interval
           a = int(num2[0].strip())
           b = int(num2[1].strip())
           if a < b: numbers.extend(range(a,b+1))
           else: numbers.extend(range(a,b-1,-1))

    return numbers


#======================================================================
# main program
#======================================================================
parser = argparse.ArgumentParser(description='Convert filband and filplot to Gnuplot format')
parser.add_argument('-f', '--filband', dest='filband', action='store', required=True,
                    help='file containing bands.x output')
parser.add_argument('-o', '--output', dest='output', action='store', required=True,
                    help='output file for Gnuplot')
parser.add_argument('-p', '--proj', dest='projlist', action='store',
                    help='list of atomic orbitals to project on')
args = parser.parse_args()

# read FILBAND    
ek = read_filband(args.filband)

# read FILPROJ
if args.projlist is not None:
    projlist = expand_list(args.projlist)
    print "projecting onto:", projlist
    proj = read_filproj(args.filband+'.proj', projlist)
    assert ek.shape == proj.shape

# write OUTPUT
print "writing file '%s' for Gnuplot" % (args.output)
f = open(args.output, "w")

nks, nbnd = ek.shape
for i in xrange(nbnd):
    f.write("# band %i\n" % (i+1))
    for ik in xrange(nks):
        if args.projlist is None:
            f.write("%4i %12.4f\n" % (ik+1, ek[ik,i]))
        else:
            f.write("%4i %12.4f %12.6f\n" % (ik+1, ek[ik,i], proj[ik,i]))
    f.write("\n")
f.close()


